﻿using UnityEngine;
using System.Collections;

public class AnimateTexture : MonoBehaviour  {
	public int materialIndex = 0;
	public Vector2 uvAnimationRate = new Vector2(0.0f, 1.0f);
	public string textureName = "_MainTex";
	public bool useRandomizedOffset = false;

	Material clonedMaterial;
	Vector2 uvOffset = Vector2.zero;

	void OnEnable() {
		this.StartCoroutine(this.DelayedRefresh());
	}
	void OnDisable() {
		this.clonedMaterial = null;
		this.StopAllCoroutines();
	}

	IEnumerator DelayedRefresh() {
		yield return new WaitForEndOfFrame();
		this.Refresh();
	}

	// call if renderer material changes
	public void Refresh() {
		if (this.GetComponent<Renderer>() != null && this.GetComponent<Renderer>().materials.Length > this.materialIndex) {
			this.clonedMaterial = this.GetComponent<Renderer>().materials[this.materialIndex];
			
			if (this.useRandomizedOffset)
				this.uvOffset = new Vector2(Random.value, Random.value);
		}
	}
	
	void LateUpdate()  {
		this.uvOffset += (this.uvAnimationRate * Time.deltaTime);
		this.uvOffset.x = this.uvOffset.x % 1;
		this.uvOffset.y = this.uvOffset.y % 1;

		if (this.clonedMaterial != null && this.GetComponent<Renderer>().enabled)
			this.clonedMaterial.SetTextureOffset(this.textureName, this.uvOffset);
	}
}
