﻿using System.Collections;
using System.Collections.Generic;
using Valve.VR.InteractionSystem;
using UnityEngine;

public class BoatMovement : MonoBehaviour
{
    private int waterLayer;
    public float acceleration = 0.1f;
    public float maxSpeed = 0.5f;

    public float angularAcceleration = 0.1f;
    public float maxAngularSpeed = 0.5f;

    private bool inWater = false;

    public float minMovementRadius = 0.5f;
    public float avoidalRadius = 1.0f;
    public float movementRadius = 10.0f;
    public float newTargetTreshold = 0.5f;

    public bool isBattleship = false;

    private new Rigidbody rigidbody;

    private Vector2 targetPosition;

    private bool isAttached = false;

    private Player player;

	// Use this for initialization
	void Start () {
        player = Player.instance;

        NewTarget();

        rigidbody = GetComponent<Rigidbody>();
        waterLayer = LayerMask.NameToLayer("Water");

        Valve.VR.InteractionSystem.Interactable interactable = GetComponent<Valve.VR.InteractionSystem.Interactable>();
        interactable.onAttachedToHand += Interactable_onAttachedToHand;
        interactable.onDetachedFromHand += Interactable_onDetachedFromHand;
    }

    private void Interactable_onDetachedFromHand(Valve.VR.InteractionSystem.Hand hand)
    {
        isAttached = false;
    }

    private void Interactable_onAttachedToHand(Valve.VR.InteractionSystem.Hand hand)
    {
        isAttached = true;
    }

    void NewTarget()
    {
        Vector2 origin = -targetPosition.normalized * (movementRadius + minMovementRadius) * 0.5f;
        float radius = (movementRadius + minMovementRadius) * 0.5f;

        targetPosition = Random.insideUnitCircle * radius + origin;

        /*targetPosition = Random.insideUnitCircle * movementRadius;
        float mag = targetPosition.magnitude;
        mag = mag / movementRadius * (movementRadius - minMovementRadius) + minMovementRadius;
        targetPosition = targetPosition.normalized * mag;*/
    }

    // Update is called once per frame
    void FixedUpdate()
    {
        if (isAttached)
        {
            return;
        }

        Vector3 rayOrigin = new Vector3(transform.position.x, transform.position.y + 0.2f, transform.position.z);

        Ray waterRay = new Ray(rayOrigin, -Vector3.up);

        RaycastHit hit;

        if (Physics.Raycast(waterRay, out hit, 0.4f, 1 << waterLayer))
        {
            inWater = true;
        }
        else
        {
            inWater = false;
        }

        if (inWater)
        {
            Vector2 shipPos = new Vector2(transform.position.x, transform.position.z);
            Vector2 toTarget = targetPosition - shipPos;
            Vector2 toTargetNorm = toTarget.normalized;
            Vector2 velocity = new Vector2(rigidbody.velocity.x, rigidbody.velocity.z);
            //float speed = Vector2.Dot(toTarget, velocity);

            //if (speed < maxSpeed)
            {
                Vector2 diff = toTargetNorm * maxSpeed - velocity;

                Vector2 force = diff.normalized * acceleration;

                Vector3 force3D = new Vector3(force.x, 0, force.y);

                Vector2 hmdPos = player.hmdTransform.position;
                hmdPos.y = 0;
                float mag = (hmdPos - shipPos).magnitude;

                float avoidal = Mathf.Clamp01((mag - minMovementRadius) / (avoidalRadius - minMovementRadius));
                Vector3 avoidalVector = new Vector3((shipPos - hmdPos).normalized.x, 0, (shipPos - hmdPos).normalized.y) * acceleration;
                
                Vector3 combined = Vector3.Slerp(avoidalVector, force3D, avoidal);

                rigidbody.AddForce(combined);
            }

            if (toTarget.magnitude < newTargetTreshold)
            {
                NewTarget();
            }

            float maxFixSpeed = maxAngularSpeed * Time.fixedDeltaTime;

            if (isBattleship)
            {
                maxFixSpeed *= 5.0f;
            }

            transform.rotation = Quaternion.RotateTowards(transform.rotation, Quaternion.LookRotation(rigidbody.velocity, Vector3.up), maxFixSpeed);
        }
    }
}
