﻿using System.Collections;
using System.Collections.Generic;
using UnityEngine;

public class Buoyancy : MonoBehaviour {
    private int waterLayer;
    private float height;

    private new Rigidbody rigidbody;

    public float force = 1.0f;
    public float damping = 5.0f;

    // Use this for initialization
    void Start()
    {
        rigidbody = GetComponent<Rigidbody>();

        waterLayer = LayerMask.NameToLayer("Water");

        Collider collider = GetComponentInChildren<Collider>();

        Vector3 size = collider.bounds.size;

        height = size.y;
    }

    // Update is called once per frame
    void FixedUpdate() {
        Vector3 rayOrigin = new Vector3(transform.position.x, 10, transform.position.z);

        Ray waterRay = new Ray(rayOrigin, -Vector3.up);

        RaycastHit hit;

        if (Physics.Raycast(waterRay, out hit, 20f, 1 << waterLayer))
        {
            float displacement = Mathf.Clamp01((hit.point.y + height/2f - transform.position.y) / height);

            rigidbody.AddForce(force * displacement * Vector3.up, ForceMode.Force);

            rigidbody.velocity *= 1f - displacement * damping * Time.fixedDeltaTime;
            rigidbody.angularVelocity *= 1f - displacement * damping * Time.fixedDeltaTime;
        }
    }
}
