﻿using System.Collections;
using System.Collections.Generic;
using UnityEngine;

[RequireComponent(typeof(MeshFilter))]
public class MeshDeformer : MonoBehaviour {
    Mesh deformingMesh;
    MeshCollider meshCollider;
    Vector3[] originalVertices, displacedVertices;
    Vector3[] vertexVelocities;

    int[][] nearbyVertices;

    public float springForce = 20.0f;
    public float verticalDamping = 5f;
    public float horizontalDamping = 10f;
    public float initialMaxVelocity = 2.0f;

    public float stickyDistance = 0.24f;
    public float stickyForce = 1.0f;

    // Use this for initialization
    void Start() {
        meshCollider = GetComponent<MeshCollider>();
        deformingMesh = GetComponent<MeshFilter>().mesh;
        originalVertices = deformingMesh.vertices;
        displacedVertices = new Vector3[originalVertices.Length];
        for (int i = 0; i < originalVertices.Length; ++i)
        {
            displacedVertices[i] = originalVertices[i];
        }

        vertexVelocities = new Vector3[originalVertices.Length];

        for (int i = 0; i < vertexVelocities.Length; ++i)
        {
            vertexVelocities[i] = Random.insideUnitSphere * initialMaxVelocity;
        }

        nearbyVertices = new int[originalVertices.Length][];

        for (int i = 0; i < nearbyVertices.Length; ++i)
        {
            int count = 0;

            for (int j = 0; j < originalVertices.Length; ++j)
            {
                if (i != j)
                {
                    float distance = (originalVertices[j] - originalVertices[i]).magnitude;

                    if (distance < stickyDistance)
                    {
                        ++count;
                    }
                }
            }
            nearbyVertices[i] = new int[count];
            int k = 0;
            for (int j = 0; j < originalVertices.Length; ++j)
            {
                if (i != j)
                {
                    float distance = (originalVertices[j] - originalVertices[i]).magnitude;

                    if (distance < stickyDistance)
                    {
                        nearbyVertices[i][k++] = j;
                    }
                }
            }
        }
    }

    public void AddDeformingForce(Vector3 point, Vector3 force)
    {
        for (int i = 0; i < displacedVertices.Length; ++i)
        {
            AddForceToVertex(i, point, force);
        }
    }

    void AddForceToVertex(int i, Vector3 point, Vector3 force)
    {
        /*Vector3 pointToVertex = displacedVertices[i] - point;
        pointToVertex *= 10.0f;
        Vector3 attenuatedForce = force / (1.0f + pointToVertex.sqrMagnitude);
        Vector3 velocity = attenuatedForce * Time.deltaTime;*/
        //vertexVelocities[i] += velocity;
        Vector3 pointToVertex = transform.TransformPoint(displacedVertices[i]) - point;
        float attenuatedForce = force.magnitude / (1f + pointToVertex.sqrMagnitude);
        float velocity = attenuatedForce * Time.deltaTime;
        vertexVelocities[i] += pointToVertex.normalized * velocity;

    }

    // Update is called once per frame
    void Update() {
        for (int i = 0; i < displacedVertices.Length; ++i)
        {
            UpdateVertexVelocity(i);
        }
        for (int i = 0; i < displacedVertices.Length; ++i)
        {
            UpdateVertexDisplacement(i);
        }
        deformingMesh.vertices = displacedVertices;
        deformingMesh.RecalculateNormals();

        meshCollider.sharedMesh = deformingMesh;
    }

    void UpdateVertexVelocity(int i)
    {
        Vector3 velocity = vertexVelocities[i];
        Vector3 displacement = displacedVertices[i] - originalVertices[i];
        velocity -= displacement * springForce * Time.deltaTime;

        float velocity_y = velocity.y;
        velocity *= 1f - horizontalDamping * Time.deltaTime;
        velocity.y = velocity_y * (1f - verticalDamping * Time.deltaTime);
        
        /*float sum = 0.0f;
        for (int j = 0; j < nearbyVertices[i].Length; ++j)
        {
            float distance = (displacedVertices[nearbyVertices[i][j]] - displacedVertices[i]).magnitude;

            if (distance < stickyDistance)
            {
                sum += displacedVertices[nearbyVertices[i][j]].y;
            }
        }

        int num = nearbyVertices[i].Length;

        float avg = sum / num;

        float diff = avg - displacedVertices[i].y;

        velocity += Vector3.up * diff * stickyForce * Time.deltaTime;*/

        vertexVelocities[i] = velocity;

    }

    void UpdateVertexDisplacement(int i)
    {
        displacedVertices[i] += vertexVelocities[i] * Time.deltaTime;
    }
}
