﻿using System.Collections;
using System.Collections.Generic;
using Valve.VR.InteractionSystem;
using UnityEngine;

public class TurretAim : MonoBehaviour {

    private Transform target;

    public Transform turret;

    public Transform verticalTurret;

    public float maxAngularSpeed = 30.0f;

    public float minAngle = -120.0f;
    public float maxAngle = 120.0f;

    public float minVerticalAngle = 0.0f;
    public float maxVerticalAngle = 90.0f;

    private float angle;
    private float targetAngle;

    private float verticalAngle;
    private float targetVerticalAngle;

    bool isAttached = false;

	// Use this for initialization
	void Start () {
        target = Player.instance.hmdTransform;

        Valve.VR.InteractionSystem.Interactable interactable = GetComponent<Valve.VR.InteractionSystem.Interactable>();
        interactable.onAttachedToHand += Interactable_onAttachedToHand;
        interactable.onDetachedFromHand += Interactable_onDetachedFromHand;
    }

    private void Interactable_onDetachedFromHand(Valve.VR.InteractionSystem.Hand hand)
    {
        isAttached = false;
    }

    private void Interactable_onAttachedToHand(Valve.VR.InteractionSystem.Hand hand)
    {
        isAttached = true;
    }
	
	// Update is called once per frame
	void Update () {
        Vector3 vectorToTarget = target.transform.position - transform.position;
        vectorToTarget.y = 0;
        Vector3 dirToTarget = vectorToTarget.normalized;
        float angleToTarget = Vector3.Angle(transform.forward, dirToTarget);

        if (Vector3.Dot(transform.right, dirToTarget) < 0.0f)
        {
            angleToTarget = -angleToTarget;
        }

        if (angleToTarget < minAngle)
        {
            targetAngle = 0.0f;
        }
        else if (angleToTarget > maxAngle)
        {
            targetAngle = 0.0f;
        }
        else
        {
            targetAngle = angleToTarget;
        }

        if (isAttached)
        {
            targetAngle = 0.0f;
        }


        if (targetAngle > angle)
        {
            angle += Mathf.Min(maxAngularSpeed * Time.deltaTime, targetAngle - angle);
        }
        else if (targetAngle < angle)
        {
            angle -= Mathf.Min(maxAngularSpeed * Time.deltaTime, angle - targetAngle);
        }

        turret.localRotation = Quaternion.Euler(turret.localRotation.eulerAngles.x, angle, turret.localRotation.eulerAngles.z);

        // vertical stuff

        Vector3 realVectorToTarget = target.transform.position - transform.position;
        float verticalAngleToTarget = Vector3.Angle(realVectorToTarget, vectorToTarget);
        
        if (verticalAngleToTarget < minVerticalAngle)
        {
            targetVerticalAngle = 45.0f;
        }
        else if (verticalAngleToTarget > maxVerticalAngle)
        {
            targetVerticalAngle = 45.0f;
        }
        else
        {
            targetVerticalAngle = 90.0f - verticalAngleToTarget;
        }

        if (isAttached)
        {
            targetVerticalAngle = 45.0f;
        }

        if (targetVerticalAngle > verticalAngle)
        {
            verticalAngle += Mathf.Min(maxAngularSpeed * Time.deltaTime, targetVerticalAngle - verticalAngle);
        }
        else if (targetVerticalAngle < verticalAngle)
        {
            verticalAngle -= Mathf.Min(maxAngularSpeed * Time.deltaTime, verticalAngle - targetVerticalAngle);
        }

        verticalTurret.localRotation = Quaternion.Euler(targetVerticalAngle, verticalTurret.localRotation.eulerAngles.y, verticalTurret.localRotation.eulerAngles.z);
    }
}
