﻿using System.Collections;
using System.Collections.Generic;
using Valve.VR.InteractionSystem;
using UnityEngine;

public class TurretShoot : MonoBehaviour {
    public Rigidbody projectilePrefab;

    public Transform spawnTransform;

    public ParticleSystem particleEffect;

    public float projectileSpeed = 10.0f;
    public float shootInterval = 5.0f;

    private float shootTimer;

    public int playerAmmo = 5;

    private Hand attachedHand;

	// Use this for initialization
	void Start () {
        shootTimer = shootInterval;

        Interactable interactable = GetComponent<Valve.VR.InteractionSystem.Interactable>();
        interactable.onAttachedToHand += Interactable_onAttachedToHand;
        interactable.onDetachedFromHand += Interactable_onDetachedFromHand;
    }

    private void Interactable_onDetachedFromHand(Valve.VR.InteractionSystem.Hand hand)
    {
        attachedHand = null;
    }

    private void Interactable_onAttachedToHand(Valve.VR.InteractionSystem.Hand hand)
    {
        attachedHand = hand;
    }
	
	// Update is called once per frame
	void Update () {
        bool shoot = false;

        if (attachedHand)
        {
            if (attachedHand.controller != null && playerAmmo > 0)
            {
                if (attachedHand.controller.GetPressDown(Valve.VR.EVRButtonId.k_EButton_SteamVR_Touchpad))
                {
                    playerAmmo--;
                    shoot = true;
                }
            }
        }
        else
        {
            shootTimer -= Time.deltaTime;

            if (shootTimer < 0.0f)
            {
                shoot = true;
                shootTimer += shootInterval;
            }
        }

        if (shoot)
        {
            Rigidbody projectile = Instantiate<Rigidbody>(projectilePrefab, spawnTransform.position, spawnTransform.rotation);

            projectile.velocity = spawnTransform.forward * projectileSpeed;
            
            Physics.IgnoreCollision(GetComponent<Collider>(), projectile.GetComponent<Collider>());

            Instantiate(particleEffect, spawnTransform.position + spawnTransform.forward * 0.1f, spawnTransform.rotation);
        }
	}
}
